#!/bin/bash

support_list="reboot|npu|memtester|stressapptest|glmark2|bt|wifi|sleep|freq|camera"
TEST_PATH=/var/log/stress-test
REBOOT_LOG=${TEST_PATH}/reboot.log
NPU_LOG=${TEST_PATH}/npu.log
MEMTESTER_LOG=${TEST_PATH}/memtester.log
STRESSAPPTEST_LOG=${TEST_PATH}/stressapptest.log
GLMARK2_LOG=${TEST_PATH}/glmark2.log
BT_LOG=${TEST_PATH}/bt.log
WIFI_LOG=${TEST_PATH}/wifi.log
SLEEP_LOG=${TEST_PATH}/sleep.log
FREQ_LOG=${TEST_PATH}/freq.log
CAMERA_LOG=${TEST_PATH}/camera.log

function help()
{
	echo "Usage: stress-test.sh [${support_list}] {HOURS}"
}

function random()
{
	echo $(expr $(hexdump -n 2 -e '/2 "%u"' /dev/random) % $1)
}

function freq_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	cpu0_available_freq=(408000 600000 816000 1008000 1200000 1416000 1608000 1800000 1920000)
	cpu4_available_freq=(408000 600000 816000 1008000 1200000 1416000 1608000 1800000 2016000 2208000 2400000)
	cpu6_available_freq=(408000 600000 816000 1008000 1200000 1416000 1608000 1800000 2016000 2208000 2400000)
	gpu_available_freq=(200000000 300000000 400000000 500000000 600000000 700000000 800000000 900000000 1000000000)
	ddr_available_freq=(528000000 1068000000 1560000000 2112000000)
	npu_available_freq=(200000000 300000000 400000000 500000000 600000000 700000000 800000000 900000000 1000000000)

	cpu0_num=${#cpu0_available_freq[@]}
	cpu4_num=${#cpu4_available_freq[@]}
	cpu6_num=${#cpu6_available_freq[@]}
	gpu_num=${#gpu_available_freq[@]}
	ddr_num=${#ddr_available_freq[@]}
	npu_num=${#npu_available_freq[@]}

	CPU_PATH=/sys/devices/system/cpu/cpufreq
	GPU_PATH=/sys/devices/platform/fb000000.gpu/devfreq/fb000000.gpu
	NPU_PATH=/sys/devices/platform/fdab0000.npu/devfreq/fdab0000.npu
	DDR_PATH=/sys/class/devfreq/dmc
	VOLT_PATH=/sys/kernel/debug/regulator

	cat $CPU_PATH/policy0/scaling_available_frequencies
	cat $CPU_PATH/policy4/scaling_available_frequencies
	cat $CPU_PATH/policy6/scaling_available_frequencies
	cat $GPU_PATH/available_frequencies
	cat $NPU_PATH/available_frequencies
	cat $DDR_PATH/available_frequencies

	echo userspace | sudo tee $CPU_PATH/policy0/scaling_governor
	echo userspace | sudo tee $CPU_PATH/policy4/scaling_governor
	echo userspace | sudo tee $CPU_PATH/policy6/scaling_governor
	echo userspace | sudo tee $GPU_PATH/governor
	echo userspace | sudo tee $NPU_PATH/governor
	echo userspace | sudo tee $DDR_PATH/governor

	echo "[FREQ] $(date): start $1h ..." | sudo tee ${FREQ_LOG}
	echo on > /sys/kernel/debug/rknpu/power
	start=$(date +%s)
	_mins=0
	while true
	do
		cpu0_freq=${cpu0_available_freq[$(random ${cpu0_num})]}
		cpu4_freq=${cpu4_available_freq[$(random ${cpu0_num})]}
		cpu6_freq=${cpu6_available_freq[$(random ${cpu0_num})]}
		gpu_freq=${gpu_available_freq[$(random ${cpu0_num})]}
		ddr_freq=${ddr_available_freq[$(random ${cpu0_num})]}
		npu_freq=${npu_available_freq[$(random ${cpu0_num})]}

		echo ${cpu0_freq}| sudo tee $CPU_PATH/policy0/scaling_setspeed
		echo ${cpu4_freq}| sudo tee $CPU_PATH/policy4/scaling_setspeed
		echo ${cpu6_freq}| sudo tee $CPU_PATH/policy6/scaling_setspeed
		echo ${gpu_freq} | sudo tee $GPU_PATH/userspace/set_freq
		echo ${ddr_freq} | sudo tee $DDR_PATH/userspace/set_freq
		echo ${npu_freq} | sudo tee $NPU_PATH/userspace/set_freq
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})
		if [ $(expr ${_secs} / 60) -gt ${_mins} ]; then
			h=$(expr ${_secs} / 3600)
			r=$(expr ${_secs} % 3600)
			m=$(expr ${r} / 60)
			s=$(expr ${r} % 60)
			_mins=$(expr ${_secs} / 60)
			echo "[FREQ] cpu0: ${cpu0_freq}, cpu4: ${cpu4_freq}, cpu6: ${cpu6_freq}, gpu: ${gpu_freq}, ddr: ${ddr_freq}, npu: ${npu_freq}" | sudo tee -a ${FREQ_LOG}
			echo "[FREQ] $(date): $h:$m:$s passed ..." | sudo tee -a ${FREQ_LOG}
		fi
		if [ ${_secs} -gt ${secs} ]; then
			sleep 1
			echo | sudo tee -a ${FREQ_LOG}
			echo "[FREQ] $(date): SUCCESS ..." | sudo tee -a ${FREQ_LOG}
			exit 0
		fi
		sleep 1
	done
	echo off > /sys/kernel/debug/rknpu/power
}

function reboot_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi
	CUSTOM=/usr/local/bin/toybrick-custom.sh
	echo "#!/bin/bash" | sudo tee ${CUSTOM}
	echo "" | sudo tee -a ${CUSTOM}
	echo "reboot.sh ${secs} >> ${REBOOT_LOG}" | sudo tee -a ${CUSTOM}

	sudo chmod +x ${CUSTOM}
	sudo reboot.sh start $1 | sudo tee ${REBOOT_LOG}
}

function camera_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	echo "[Camera] $(date): start $1h ..." | sudo tee ${CAMERA_LOG}
	toycam -d $(toycam -l | grep mipi |awk -F'video' '{print $2}') &
	start=$(date +%s)
	while true
	do
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})
		if [ ${_secs} -gt ${secs} ]; then
			sudo pkill -9 toycam
			sleep 1
			echo | sudo tee -a ${CAMERA_LOG}
			echo "[Camera] $(date): SUCCESS ..." | sudo tee -a ${CAMERA_LOG}
			exit 0
		fi
		sleep 1
	done
}

function memtester_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	ddr_cap=$(expr $(cat /proc/meminfo | grep MemTotal | awk -F' ' '{print $2}') / 1000)
	if [ ${ddr_cap} -ge 15500 ]; then
		memtester_cap=2048m
	elif [ ${ddr_cap} -ge 7500 ]; then
		memtester_cap=1024m
	elif [ ${ddr_cap} -ge 3500 ]; then
		memtester_cap=512m
	elif [ ${ddr_cap} -ge 1500 ]; then
		memtester_cap=256m
	else
		memtester_cap=128m
	fi

	echo "[Memtester] $(date): start $1h ..." | sudo tee ${MEMTESTER_LOG}
	sudo memtester ${memtester_cap} 2>&1 | sudo tee -a ${MEMTESTER_LOG} &
	start=$(date +%s)
	while true
	do
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})
		if [ ${_secs} -gt ${secs} ]; then
			sudo pkill -9 memtester
			sleep 1
			echo | sudo tee -a ${MEMTESTER_LOG}
			echo "[Memtester] $(date): SUCCESS ..." | sudo tee -a ${MEMTESTER_LOG}
			exit 0
		fi
		sleep 1
	done
}

function stressapptest_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	ddr_cap=$(expr $(cat /proc/meminfo | grep MemTotal | awk -F' ' '{print $2}') / 1000)
	if [ ${ddr_cap} -ge 15500 ]; then
		stressapptest_cap=4096
	elif [ ${ddr_cap} -ge 7500 ]; then
		stressapptest_cap=2048
	elif [ ${ddr_cap} -ge 3500 ]; then
		stressapptest_cap=1024
	elif [ ${ddr_cap} -ge 1500 ]; then
		stressapptest_cap=512
	else
		stressapptest_cap=256
	fi

	echo "[Stessapptest] $(date): start $1h ..." | sudo tee ${STRESSAPPTEST_LOG}
	sudo stressapptest -s ${secs} -i 4 -C 4 -W --stop_on_errors -M ${stressapptest_cap} | sudo tee -a ${STRESSAPPTEST_LOG}
	sleep 1
	echo | sudo tee -a ${STRESSAPPTEST_LOG}
	if [ $? -eq 0 ]; then
		echo "[Stessapptest] $(date): SUCCESS ..." | sudo tee -a ${STRESSAPPTEST_LOG}
	else
		echo "[Stessapptest] $(date): FAILED ..." | sudo tee -a ${STRESSAPPTEST_LOG}
	fi
}

function glmark2_test()
{
	echo "[Glmark2] $(date): start $1h ..." | sudo tee ${GLMARK2_LOG}
	if [ $1 -eq 0 ]; then
		glmark2 | sudo tee -a ${GLMARK2_LOG} 
		echo "[Glmark2] $(date): SUCCESS ..." | sudo tee -a ${GLMARK2_LOG}
		exit 0
	fi

	secs=$(expr $1 \* 3600)
	glmark2 --run-forever 2>&1 | sudo tee -a ${GLMARK2_LOG} &
	start=$(date +%s)
	_mins=0
	while true
	do
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})	
		if [ $(expr ${_secs} / 60) -gt ${_mins} ]; then
			h=$(expr ${_secs} / 3600)
			r=$(expr ${_secs} % 3600)
			m=$(expr ${r} / 60)
			s=$(expr ${r} % 60)
			_mins=$(expr ${_secs} / 60)
			echo | sudo tee -a ${GLMARK2_LOG}
			echo "[Glmark2] $(date): $h:$m:$s passed ..." | sudo tee -a ${GLMARK2_LOG}
		fi
		if [ ${_secs} -gt ${secs} ]; then
			sudo pkill -9 glmark2
			sleep 1
			echo | sudo tee -a ${GLMARK2_LOG}
			echo "[Glmark2] $(date): SUCCESS ..." | sudo tee -a ${GLMARK2_LOG}
			exit 0
		fi
		sleep 1
	done
}

function npu_test()
{
	secs=$(expr $1 \* 3600)

	echo "[NPU] $(date): start $1h ..." | sudo tee ${NPU_LOG}
	cd /usr/local/npu2test
	./npu2test.sh ${secs} 2>&1 | sudo tee -a ${NPU_LOG}
	cd -
	echo | sudo tee -a ${NPU_LOG}
	if [[ x"$(cat ${NPU_LOG} | grep FAILED)" == x"" ]] && [[ x"$(cat ${NPU_LOG} | grep Aborted)" == x"" ]]; then
		echo "[NPU] $(date): SUCCESS ..." | sudo tee -a ${NPU_LOG}
	else
		echo "[NPU] $(date): FAILED ..." | sudo tee -a ${NPU_LOG}
	fi
}

function wifi_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	echo "[WIFI] $(date): start $1h ..." | sudo tee ${WIFI_LOG}
	start=$(date +%s)
	_mins=0
	while true
	do
		sudo ifconfig wlan0 down
		sleep 1
		sudo ifconfig wlan0 up
		sleep 1
		if [ ! -n "$(sudo ifconfig wlan0 | grep "UP,BROADCAST,MULTICAST")" ]; then
			echo "[WIFI] $(date): FAILED ..." | sudo tee -a ${WIFI_LOG}
			exit 1
		fi
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})
		if [ $(expr ${_secs} / 60) -gt ${_mins} ]; then
			h=$(expr ${_secs} / 3600)
			r=$(expr ${_secs} % 3600)
			m=$(expr ${r} / 60)
			s=$(expr ${r} % 60)
			_mins=$(expr ${_secs} / 60)
			echo "[WIFI] $(date): $h:$m:$s passed ..." | sudo tee -a ${WIFI_LOG}
		fi
		if [ ${_secs} -gt ${secs} ]; then
			sleep 1
			echo | sudo tee -a ${WIFI_LOG}
			echo "[WIFI] $(date): SUCCESS ..." | sudo tee -a ${WIFI_LOG}
			exit 0
		fi
		sleep 1
	done
}

function bt_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	echo "[BT] $(date): start $1h ..." | sudo tee ${BT_LOG}
	start=$(date +%s)
	_mins=0
	while true
	do
		sudo rfkill block bluetooth
		sleep 1
		sudo rfkill unblock bluetooth
		sleep 1
		if [ ! -n "$(sudo hciconfig hci0 | grep "UP RUNNING")" ]; then
			echo "[BT] $(date): FAILED ..." | sudo tee -a ${BT_LOG}
			exit 1
		fi
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})
		if [ $(expr ${_secs} / 60) -gt ${_mins} ]; then
			h=$(expr ${_secs} / 3600)
			r=$(expr ${_secs} % 3600)
			m=$(expr ${r} / 60)
			s=$(expr ${r} % 60)
			_mins=$(expr ${_secs} / 60)
			echo "[BT] $(date): $h:$m:$s passed ..." | sudo tee -a ${BT_LOG}
		fi
		if [ ${_secs} -gt ${secs} ]; then
			sleep 1
			echo | sudo tee -a ${BT_LOG}
			echo "[BT] $(date): SUCCESS ..." | sudo tee -a ${BT_LOG}
			exit 0
		fi
		sleep 1
	done
}

function sleep_test()
{
	if [ $1 -eq 0 ]; then
		secs=60
	else
		secs=$(expr $1 \* 3600)
	fi

	sleep_time=10

	# set sys time same with rtc
	sudo hwclock --systohc
	sudo hwclock -w
	echo "[SLEEP] $(date): start $1h ..." | sudo tee ${SLEEP_LOG}
	start=$(date +%s)
	_mins=0
	while true
	do
		echo 0 | sudo tee /sys/class/rtc/rtc0/wakealarm
		echo "+${sleep_time}" | sudo tee /sys/class/rtc/rtc0/wakealarm
		sudo pm-suspend
		stop=$(date +%s)
		_secs=$(expr ${stop} - ${start})
		if [ $(expr ${_secs} / 60) -gt ${_mins} ]; then
			h=$(expr ${_secs} / 3600)
			r=$(expr ${_secs} % 3600)
			m=$(expr ${r} / 60)
			s=$(expr ${r} % 60)
			_mins=$(expr ${_secs} / 60)
			echo "[SLEEP] $(date): $h:$m:$s passed ..." | sudo tee -a ${SLEEP_LOG}
		fi
		if [ ${_secs} -gt ${secs} ]; then
			sleep 1
			echo | sudo tee -a ${SLEEP_LOG}
			echo "[SLEEP] $(date): SUCCESS ..." | sudo tee -a ${SLEEP_LOG}
			exit 0
		fi
		sleep ${sleep_time}
	done
}

if [ $# -lt 1 ]; then
	help
	exit 1
fi

sudo mkdir -p ${TEST_PATH}

case $1 in
reboot|npu|memtester|stressapptest|glmark2|bt|wifi|sleep|freq|camera)
	$1_test $2
	;;
*)
	help
	exit 1
	;;
esac


